#!/bin/sh
#\
exec tclsh "$0" "$@"

# Written by Falk Richter
#
# tk_msg (for a short description please see the About-information) 
#
# This software stands under the GNU GENERAL PUBLIC LICENSE and comes with 
# NO warranty of any kind. Use at your own risk.
# Report bugs or suggestions to: please see contact in the about-box
# ===============================================================================


namespace eval ::tk_msg \
{
  namespace export Main
}


# we load the msgcat package in a global scope  
package require msgcat

# we check if a language is set from the operation system
if {![info exist env(LANG)]} {set env(LANG) "en"}
msgcat::mclocale $env(LANG)
msgcat::mcload [file join [pwd] msgs]


#---the mother of all procedures :o)
proc ::tk_msg::Main {} \
{
  variable CONFIG; variable VAR
 
  ::tk_msg::Startup
 
  wm protocol . WM_DELETE_WINDOW ::tk_msg::Exit

  ::tk_msg::Frames 
 
  ::tk_msg::Gui
 
  ::tk_msg::Menu
 
  #::tk_msg::BalloonHelp
 
  #::tk_msg::FirstStart
 
  wm geometry . $CONFIG(Geometry)
  wm deiconify .
  update
}



#---here we load all need packages, set the defaults, read and parse the config file
proc ::tk_msg::Startup {} \
{
  variable VAR; variable ABOUT;
  variable CONFIGFILE; variable DEFAULTS; variable CONFIG;
  global env auto_path tcl_platform
 
  catch {package require Tk}
  wm withdraw .
 
  set VAR(NameSpace) [namespace current]
  set VAR(StartDir) [pwd]
 
  # we check the existens of lib directory   
  set VAR(LibDir) [file join $VAR(StartDir) lib]
 
  if { [file exists $VAR(LibDir)] !="1" } \
  {
    tk_messageBox -message [msgcat::mc "I can't find the program libraries!\n\nPlease install the program again."]\
    -title [msgcat::mc "Where are my libraries?"] -icon error -type ok
    destroy .
    exit
  } else \
  {
    lappend auto_path $VAR(LibDir)
  }
 
  # we load all needed packages
  ::tk_msg::Packageload [list FR_messageBox FR_config FR_general FR_selectBox FR_reverseDirContent]
  
  # the informations for the about boxes and the window border
  set InfoFile [file join $VAR(StartDir) ".info.tcl"]
 
  if {[file exists $InfoFile]} \
  {
    source $InfoFile
  } else \
  {
    tk_messageBox -message [msgcat::mc "I can't find the file:\n\n\"%s\"\n\nPlease install the program again." $InfoFile]\
    -title [msgcat::mc "File not found"] -icon error -type ok
    destroy .
    exit
  }
 
  # some settings for the toplevel windows
  set VAR(Parent) .
  wm title $VAR(Parent) "$ABOUT(Name) version $ABOUT(Version) from $ABOUT(Date)"
  wm protocol $VAR(Parent) WM_DELETE_WINDOW ::tk_msg::Exit
  
  # the default settings
  #set DEFAULTS(BALLOON_HELP) "1"
  set DEFAULTS(Color) "sandyellow"
  set DEFAULTS(Geometry) "+20+20"
  set DEFAULTS(SourceDir) $VAR(StartDir)
 
  # we read and parse the configuration file
  set ConfigFileDir [file join $env(HOME) .${ABOUT(Name)}]
 
  if {![file exists $ConfigFileDir]} \
  {
    catch {file mkdir $ConfigFileDir}
  }
 
  set CONFIGFILE [file join $ConfigFileDir .${ABOUT(Name)}_conf.tcl]
 
  ::FR_config::ReadConfig $VAR(NameSpace)

  # we check the config directories and set to the default values if the directory doesn't exist
  foreach Arg {SourceDir} \
  {
    if { ![file isdirectory $CONFIG($Arg)] } \
    {
      set CONFIG($Arg) $DEFAULTS($Arg)
    } 
  }
 
  # we define the "look" (colours)
  set OptionsFile [file join $VAR(StartDir) .tk_options.tcl]
 
  if {[file exists $OptionsFile]} \
  {
    source $OptionsFile
    database_options $VAR(NameSpace)
  }
  
  set VAR(Changed) 0
  
  # namespace variable to save the unused messages / translations
  set VAR(CatMsgList) ""
  set VAR(CatTranslationList) ""
}


#---the package loader
#---we load the needed package only one time
proc ::tk_msg::Packageload {Packages} \
{
  variable VAR

  foreach Arg $Packages \
  {
    if {![info exists VAR($Arg)]} \
    {
      if { [catch {package require $Arg} output] !="0" } \
      {
        tk_messageBox -message [msgcat::mc "I can't load this part from the program library:\n\n%s\n\nPlease install the program again." $Arg]\
        -title [msgcat::mc "Parts of the program couldn't be loaded"] -icon error -type ok
        destroy .
        exit
      } else \
      {
        set VAR($Arg) 1
      }
    }
  }
}


#---loads an external proc (and read the file if necessary)
proc ::tk_msg::LoadProc {Procedure File} \
{
  variable VAR
 
  #set Procedure [string trimleft $Procedure [namespace current]]
  #set Procedure [string trimleft $Procedure ::]
  
  if {[info procs $Procedure] eq ""} \
  {
    source [file join $VAR(StartDir) tcl $File]
  }
}


#---we save the configuration and finish the program
proc ::tk_msg::Exit {} \
{
  variable CONFIG; variable VAR
 
  ::FR_config::SaveConfig $VAR(NameSpace)
 
  catch {puts [msgcat::mc "Goodby!"]}
  destroy .
  exit
}


#---We create the frames and set a variable for each frame - 
#---so we can change the order of the frames later
proc ::tk_msg::Frames {} \
{
  variable VAR
 
  frame .frame1 -relief flat
  labelframe .frame2 -text [msgcat::mc "First choose your files/directories"] -relief groove -padx 5 -pady 5
  frame .frame3 -relief flat
  frame .frame4 -relief flat

  pack .frame1 -side top -fill x -padx 5
  pack .frame2 -side top -fill x -padx 5 -pady 5
  pack .frame3 -side top -fill both -expand yes -padx 5  
  pack .frame4 -side bottom -fill x -padx 5
     
  set VAR(menuframe) "frame1"
  set VAR(choiceframe) "frame2"
  set VAR(mainframe) "frame3"
  set VAR(bottomframe) "frame4"
}


#---This create the menu
proc ::tk_msg::Menu {} \
{
  variable VAR
   
  menubutton .$VAR(menuframe).file -text [msgcat::mc "File"] -menu .$VAR(menuframe).file.choice -underline 0
  menubutton .$VAR(menuframe).edit -text [msgcat::mc "Edit"] -menu .$VAR(menuframe).edit.choice -underline 0
  menubutton .$VAR(menuframe).about -text [msgcat::mc "About"] -menu .$VAR(menuframe).about.choice -underline 0

  menu .$VAR(menuframe).file.choice -tearoff 0
  .$VAR(menuframe).file.choice add command -label [msgcat::mc "Exit"] -command {::tk_msg::Exit} -accelerator "Ctr+q"
 
  
  menu .$VAR(menuframe).edit.choice -tearoff 0
  .$VAR(menuframe).edit.choice add command -label [msgcat::mc "Open unused Translations"] -command {::tk_msg::UnusedTranslations} -accelerator "Ctr+o"
  
  menu .$VAR(menuframe).about.choice -tearoff 0
  .$VAR(menuframe).about.choice add command -label [msgcat::mc "About"] -accelerator "Alt+a" -command [list ::FR_messageBox::Create -type About -nspace $VAR(NameSpace)]
 
  pack .$VAR(menuframe).file -side left
  pack .$VAR(menuframe).edit -side left
  pack .$VAR(menuframe).about -side right
 
  bind . <Control-KeyPress-q> {::tk_msg::Exit}
  bind .$VAR(menuframe).file.choice <Control-KeyPress-q> [list ::FR_general::MenuBinding .$VAR(menuframe).file.choice 0]
  bind .$VAR(menuframe).edit.choice <Control-KeyPress-o> [list ::FR_general::MenuBinding .$VAR(menuframe).edit.choice 0]
  bind .$VAR(menuframe).about.choice <Alt-KeyPress-a> [list ::FR_general::MenuBinding .$VAR(menuframe).about.choice 0]
}


#---we create all widgets
proc ::tk_msg::Gui {} \
{
  variable VAR; variable CONFIG
  
  label .$VAR(choiceframe).source_comment -text [msgcat::mc "The directory with the source code:"]
  label .$VAR(choiceframe).msgcat_comment -text [msgcat::mc "Open existing message catalogues:"]
  label .$VAR(choiceframe).msgcat_new_comment -text [msgcat::mc "The location for the new message catalogue:"]

  button .$VAR(choiceframe).source -text [msgcat::mc "Open"] -command {::tk_msg::OpenSource}
  button .$VAR(choiceframe).msgcat -text [msgcat::mc "Open"] -command {::tk_msg::OpenMsgcatFiles} -state disable
  button .$VAR(choiceframe).msgcat_new -text [msgcat::mc "Save as"] -command {::tk_msg::CreateMsgcatFile} -state disable
  
  grid .$VAR(choiceframe).source_comment -row 0 -column 0 -sticky w
  grid .$VAR(choiceframe).source -row 0 -column 2 -sticky ew

  grid .$VAR(choiceframe).msgcat_comment -row 1 -column 0 -sticky w
  grid .$VAR(choiceframe).msgcat -row 1 -column 2 -sticky ew

  grid .$VAR(choiceframe).msgcat_new_comment -row 2 -column 0 -sticky w
  grid .$VAR(choiceframe).msgcat_new -row 2 -column 2 -sticky ew

  grid columnconfigure .$VAR(choiceframe) 1 -weight 1


  label .$VAR(mainframe).comment1 -text [msgcat::mc "The orginal messages"]
  label .$VAR(mainframe).comment2 -text [msgcat::mc "Translation"]
  
  set VAR(ListBoxOrginalPath) $VAR(mainframe).msg_files
  set VAR(ListBoxTranslationPath) $VAR(mainframe).msg_cat
  
  listbox .$VAR(ListBoxOrginalPath) -background white -activestyle none -selectmode browse \
  -yscroll ".$VAR(mainframe).yscrollbar set" -xscroll ".$VAR(mainframe).xscrollbar1 set"

  listbox .$VAR(ListBoxTranslationPath) -background white -activestyle none -selectmode browse \
  -yscroll ".$VAR(mainframe).yscrollbar set" -xscroll ".$VAR(mainframe).xscrollbar2 set"
   
  scrollbar .$VAR(mainframe).yscrollbar -orient vertical -command {::tk_msg::MultiScroll}
  scrollbar .$VAR(mainframe).xscrollbar1 -orient horizontal -command ".$VAR(ListBoxOrginalPath) xview"
  scrollbar .$VAR(mainframe).xscrollbar2 -orient horizontal -command ".$VAR(ListBoxTranslationPath) xview"

  button .$VAR(mainframe).write -text [msgcat::mc "Write the message cataloge"] -command {::tk_msg::WriteFile} -state disable

  grid .$VAR(mainframe).comment1 -row 0 -column 0 -sticky w -padx {5 0} -pady 5
  grid .$VAR(mainframe).comment2 -row 0 -column 1 -sticky w -padx {0 5} -pady 5
  grid .$VAR(ListBoxOrginalPath) -row 1 -column 0 -sticky nsew -padx {5 0}
  grid .$VAR(ListBoxTranslationPath) -row 1 -column 1 -sticky nsew -padx 0
  grid .$VAR(mainframe).yscrollbar -row 1 -column 2 -sticky ns -padx {0 5} -pady {5 0}
  grid .$VAR(mainframe).xscrollbar1 -row 2 -column 0 -sticky ew -padx {5 0} -pady {0 5}
  grid .$VAR(mainframe).xscrollbar2 -row 2 -column 1 -sticky ew -padx 0 -pady {0 5}

  grid .$VAR(mainframe).write -row 3 -column 0 -columnspan 3 -sticky ew -padx 5 -pady 5

  grid columnconfigure .$VAR(mainframe) 0 -weight 1
  grid columnconfigure .$VAR(mainframe) 1 -weight 1
  grid rowconfigure .$VAR(mainframe) 1 -weight 1

  bind .$VAR(ListBoxOrginalPath) <<ListboxSelect>> {::tk_msg::MsgUsedIn %W}
  bind .$VAR(ListBoxTranslationPath) <<ListboxSelect>> {::tk_msg::EditEntry}

  bind .$VAR(ListBoxOrginalPath) <ButtonPress-4> {::tk_msg::MultiScroll scroll -1 units; break}
  bind .$VAR(ListBoxTranslationPath) <ButtonPress-4> {::tk_msg::MultiScroll scroll -1 units; break}
  bind .$VAR(ListBoxOrginalPath) <ButtonPress-5> {::tk_msg::MultiScroll scroll 1 units; break}
  bind .$VAR(ListBoxTranslationPath) <ButtonPress-5> {::tk_msg::MultiScroll scroll 1 units; break}
}


#---proc to scroll muliple listboxes with one scrollbar
proc ::tk_msg::MultiScroll {args} \
{  
  variable VAR
  
  foreach i [list .$VAR(ListBoxOrginalPath) .$VAR(ListBoxTranslationPath)] \
  {
    eval {$i yview} $args
  }
}


#---the proc to choice the source directory
proc ::tk_msg::OpenSource {} \
{
  variable VAR; variable CONFIG
 
  set SourceDir [::FR_dirBox::Create -initialdir $CONFIG(SourceDir) -title [msgcat::mc "Please choose your source directory:"]]
 
  if {[llength $SourceDir] == 0} \
  {
    return 1
  }
 
  set CONFIG(SourceDir) $SourceDir
 
  set VAR(MsgcatFiles) ""
  set VAR(NewMsgcatFile) ""
  
  .$VAR(choiceframe).msgcat configure -state normal
  .$VAR(choiceframe).msgcat_new configure -state normal
  .$VAR(mainframe).write configure -state disable
  
  ::tk_msg::DeleteListBox
}


#---the proc to choose the msgcat file
proc ::tk_msg::OpenMsgcatFiles {} \
{
  variable VAR
  variable CONFIG
 
  set MsgcatFilesDir [list [file join [lindex $CONFIG(SourceDir) 0] msgs]]
 
  if {![file exists [lindex $MsgcatFilesDir 0]]} \
  {
    set MsgcatFilesDir $CONFIG(SourceDir)
  }
 
  set MsgcatFiles [::FR_selectBox::Create -type Files -title [msgcat::mc "Please choose your msgcat file(s):"] -initialdir $MsgcatFilesDir -initialfile "de.msg" -only_filetype 1 -filetype [list [msgcat::mc "Message cataloge files"] "*.msg"] -preload $VAR(MsgcatFiles) -multiple 1]

  if {$MsgcatFiles eq ""} \
  {
    return 1
  }
  
  set MsgcatFilesLength [llength $MsgcatFiles]
  
  # if you choose more than one file, all files must have the same file name
  if {$MsgcatFilesLength > 1} \
  {
    for {set i 0} {$i < $MsgcatFilesLength} {incr i} \
    {
      set Arg [lindex $MsgcatFiles $i]
      set Arg [file tail $Arg]
    
      # in the first loop we store the file name first element
      if {$i == 0} \
      {
        set FirstElement $Arg
      } \
      else \
      {
        # the file names must be always the same
        if {$Arg ne $FirstElement} \
        {
          ::FR_messageBox::Create -type Error -message [msgcat::mc "You can't mix msgcat's of different languages"]
          return 1
        }
      }
    }
  }
 
  set VAR(MsgcatFiles) $MsgcatFiles
 
  # if only one msgcat file was opened so we assume you want to write your changes in the same file
  if {$MsgcatFilesLength == 1} \
  {
    set Answer [::FR_messageBox::Create -message [msgcat::mc "You have chosen one message catalogue.\n\nWould you save your changes in the same file?"] -title [msgcat::mc "Question?"] -type Info -button [list [msgcat::mc "Yes"] [msgcat::mc "No"]]]
    
    if {$Answer} \
    {
      set VAR(NewMsgcatFile) $VAR(MsgcatFiles)
      .$VAR(mainframe).write configure -state normal
      ::tk_msg::ParseFiles
    } \
    else \
    {
      set VAR(NewMsgcatFile) ""
      .$VAR(mainframe).write configure -state disable
    }
  } 
}


#---the proc to create a new empty msgcat file
proc ::tk_msg::CreateMsgcatFile {} \
{
  variable VAR; variable CONFIG
 
  set NewMsgcatFileDir [file join $CONFIG(SourceDir) msgs] 
 
  if {![file exists $NewMsgcatFileDir]} \
  {
    set NewMsgcatFileDir $CONFIG(SourceDir)
  }
  
  set NewMsgcatFile [::FR_fileBox::Create -type "Save_as" -initialdir $NewMsgcatFileDir -only_filetype 1 -filetype [list [msgcat::mc "Message cataloge files"] "*.msg"] -permission "writeable"]
  
  if {$NewMsgcatFile eq ""} \
  {
    return 1
  }
 
  set VAR(NewMsgcatFile) $NewMsgcatFile
 
  # we start the main proc: the comparision between source code and allready done translations (msgcat-file)-----
  ::tk_msg::ParseFiles
 
  .$VAR(mainframe).write configure -state normal
}


#---this binding visite the file which contents this message/text
proc ::tk_msg::MsgUsedIn {widget} \
{
  variable FilesMsgLocation
 
  set Index [$widget curselection]
  
  # this avoids an error if we click in a empty listbox
  if { $Index eq "" } \
  {
    return
  }
 
  set Content [$widget get $Index]
 
 ::FR_messageBox::Create -message [msgcat::mc "This message comes from:\n\n%s" $FilesMsgLocation($Content)]
}


#---this creates a 2nd window for the translation input
proc ::tk_msg::EditEntry {} \
{
  variable VAR
 
  set Index [.$VAR(ListBoxTranslationPath) curselection]
  
  # this avoids an error if we click in a empty listbox
  if { $Index eq "" } \
  {
    return
  }
 
  set Translation [.$VAR(ListBoxTranslationPath) get $Index]
  set Message [.$VAR(ListBoxOrginalPath) get $Index]
 
  ::tk_msg::LoadProc EditEntrySub "edit_entry.tcl"
 
  set ReturnValue [::tk_msg::EditEntrySub $Message $Translation]

  if {$ReturnValue ne ""} \
  {
    set VAR(Changed) 1
    .$VAR(ListBoxTranslationPath) delete $Index
    .$VAR(ListBoxTranslationPath) insert $Index $ReturnValue
  }
}


#---this make a copy of the old xx.msg file a write the new one (only if anything changed)
proc ::tk_msg::WriteFile {} \
{
  variable VAR

  # we check if anything changed
  if {!$VAR(Changed)} \
  {
    ::FR_messageBox::Create -message [msgcat::mc "We didn't write a file because you hadn't changed anything yet."]
    return
  }
 
  set BackupFile ""
  append BackupFile [file rootname $VAR(NewMsgcatFile)] .bak
 
  file copy -force $VAR(NewMsgcatFile) $BackupFile
 
  set CatchReturn [catch {set Fid [open $VAR(NewMsgcatFile) w]} Error]
  
  if {$CatchReturn} \
  {
    ::FR_messageBox::Create -type Error -message [msgcat::mc "I can't open: %s\nThe reason for this was: %s" $VAR(NewMsgcatFile) $Error]
    return 1
  }
  
  puts $Fid "#---This is a message cataloge generated with tk_msg.tcl"
 
  set Messages [.$VAR(ListBoxOrginalPath) get 0 end]
  set Translation [.$VAR(ListBoxTranslationPath) get 0 end]
 
  foreach i $Messages j $Translation \
  {
    puts $Fid "::msgcat::mcset $VAR(Language) \"$i\" \"$j\""
  }
 
  close $Fid
 
  ::FR_messageBox::Create -message [msgcat::mc "Msg-file was successful wrote."]
  set VAR(Changed) 0
}


#---clear both listboxes
proc ::tk_msg::DeleteListBox {} \
{
  variable VAR
  
  .$VAR(ListBoxOrginalPath) delete 0 end
  .$VAR(ListBoxTranslationPath) delete 0 end
}


#---proc to fill a listbox
proc ::tk_msg::FillListBox {Path Content}\
{ 
  foreach i $Content \
  {
    .$Path insert end $i
  }
}


#---the main proc: the comparision of xx.msg and all found [msgcat mc "xxx"] constructs
proc ::tk_msg::ParseFiles {} \
{
  variable VAR; variable CONFIG
  variable FilesMsgLocation
  
  # namespace variable to save the unused messages / translations
  set VAR(CatMsgList) ""
  set VAR(CatTranslationList) ""
  
  # we open and read out the xx.msg files
  set MsgcatFilesContent ""
  
  foreach Arg $VAR(MsgcatFiles) \
  {
    set CatchReturn [catch {set Fid [open $Arg r]} Error]
    
    if {$CatchReturn} \
    {
      ::FR_messageBox::Create -type Error -message [msgcat::mc "I can't open: %s\nThe reason for this was: %s" $Arg $Error]
      return 1
    }
    
    append MsgcatFilesContent [read $Fid]
    close $Fid
  }
 
  # we find the language in the name of the new msgcat file
  set VAR(Language) [file rootname [file tail $VAR(NewMsgcatFile)]]

  # we read each line and sort out all none msgcat lines
  set MsgcatFilesContent [regexp -all -inline -- {::msgcat::mcset [^\n]+} $MsgcatFilesContent]
 
  # CatMsgList is the list of all existing messages in the xx.msg file
  set CatMsgList ""
  # CatMsgListMapped is the same as CatMsgList but without any \n
  set CatMsgListMapped ""
  
  # CatTranslationList is the list of all existing translations in the xx.msg file
  set CatTranslationList ""
 
  # we separate messages and translations (form the xx.msg file)
  foreach i $MsgcatFilesContent \
  {
    set Cut1 [string first {"} $i]
    set Cut1 [expr {${Cut1} + 1}]
  
    set Cut2 [string last {" "} $i]
    set Cut2 [expr {${Cut2} - 1}]
  
    set Cut3 [expr {${Cut2} + 4}]
  
    set Cut4 [string last {"} $i]
    set Cut4 [expr {${Cut4} - 1}]
  
    set CatMsg [string range $i $Cut1 $Cut2]
  
    set CatTranslation [string range $i $Cut3 $Cut4] 
    
    lappend CatMsgList $CatMsg
    eval set CatMsgMapped \"[string map {\\n ///n} $CatMsg]\"
    lappend CatMsgListMapped $CatMsgMapped
    lappend CatTranslationList $CatTranslation
  }  

  unset MsgcatFilesContent
  
  set SourceFiles [::FR_reverseDirContent::Start $CONFIG(SourceDir) 0 1 "*.tcl"]
 
  # we read the content of each source file
  foreach SourceFile $SourceFiles \
  { 
    set SourceFilesContent($SourceFile) [::tk_msg::ReadFile $SourceFile]
  }
 
  # FilesMsgList is the list of all existing messages in the source files
  set FilesMsgList ""
 
  # we filter out the message from the [msgcat::mc "blah, blah"] construct
  foreach SourceFile $SourceFiles \
  {
    set SearchResult [regexp -all -inline -- {\[(\s)?(msgcat::mc|::msgcat::mc|mc)(\s)+[^\]]+} $SourceFilesContent($SourceFile)]
  
    set SearchResultLength [llength $SearchResult]
  
    if {$SearchResultLength > 0} \
    {
      for {set Index 0} {$Index < $SearchResultLength} {incr Index 4} \
      {
        set j [lindex $SearchResult $Index]
    
        set Cut1 [string first {"} $j]
    
        if {$Cut1 != -1} \
        { 
          set Cut2 [string last {"} $j]
     
          # if first and last quotation marks the same: there is only one quotation mark
          if {$Cut1 == $Cut2} \
          {
            if {[string index $j end] eq "\\"} \
            {
              ::FR_messageBox::Create -type "Error" -message [msgcat::mc "You are using in your orginal message:\n\n\"%s\"\n\nthe successive characters: %s\n\nUnconfortable tk_msg can't deal with this." $j {\]}] -title [msgcat::mc "Detect a problem"]
              return 1
            } \
            else \
            {
              ::FR_messageBox::Create -type "Error" -message [msgcat::mc "I can't find the closing \" in your orginal message:\n\n\"%s\"\n\nUnconfortable tk_msg can't deal with this." $j] -title [msgcat::mc "Detect a problem"]
              return 1
            }
          } \
          else \
          {
            # the normal case
            set Cut1 [expr {${Cut1} + 1}]
            set Cut2 [expr {${Cut2} - 1}]
    
            set Message [string range $j $Cut1 $Cut2]
    
            set FilesMsgLocation($Message) $SourceFile
    
            lappend FilesMsgList $Message
          }
        } 
      }
    }
  }
 
 
  # we sort the orginal messages alphapetical and filter out all doubles
  set FilesMsgList [lsort -unique -asci $FilesMsgList] 
 
  # we are looking for all already existing translations
  set TranslationList ""
  
  # all indecies of the msgcat translations which are used in the source files
  set IndexList ""
  
  foreach i $FilesMsgList \
  {
    # if the Msg contains a "\n" lsearch doesn't find the pattern
    # so we are using the mapped version (\n is replace with ///n) of the CatMsgList
    
    eval set MessageMapped \"[string map {\\n ///n} $i]\"
    
    set IndexAll [lsearch -all -exact $CatMsgListMapped $MessageMapped]
    set Index [lindex $IndexAll 0]
    
    if { $Index ne ""} \
    {
      lappend TranslationList [lindex $CatTranslationList $Index]
      set IndexList [concat $IndexList $IndexAll]
    } \
    else \
    {
      lappend TranslationList ""
    }
  }
 
  # we fill the listboxes
  ::tk_msg::DeleteListBox
   
  ::tk_msg::FillListBox $VAR(ListBoxOrginalPath) $FilesMsgList
 
  ::tk_msg::FillListBox $VAR(ListBoxTranslationPath) $TranslationList

  # all unused translations for the msgcat files are saved in a text file
  set IndexList [lsort -integer -decreasing $IndexList]
  
  foreach i $IndexList \
  {
    set CatMsgList [lreplace $CatMsgList $i $i]
    set CatTranslationList [lreplace $CatTranslationList $i $i]  
  }
  
  if {[llength $CatTranslationList] > 0} \
  {
    set VAR(CatMsgList) $CatMsgList
    set VAR(CatTranslationList) $CatTranslationList
    unset CatMsgList
    unset CatTranslationList
  } 
}


#---this read out the content of all files and puts all broken down lines in one line
proc ::tk_msg::ReadFile {File} \
{
  set CommentLine 0
  set FileContent ""
 
  set CatchReturn [catch {set Fid [open $File r]} Error]
  
  if {$CatchReturn} \
  {
    ::FR_messageBox::Create -type Error -message [msgcat::mc "I can't open: %s\nThe reason for this was: %s" $File $Error]
    return 1
  }
  
  while {[gets $Fid CurrLine] >= 0} \
  {
    # we delete the leading and ending whitespaces in each line
    set CurrLine [string trim $CurrLine]
    set LineBreak [expr {[string index $CurrLine end] eq "\\"}]
    
    # If the first character is a comment, we are looking after break down character
    if {[regexp {^(#)+} $CurrLine] == 1} \
    {
      if {$LineBreak} \
      {
        set CommentLine 1
      }  
      continue
    }
  
    # we overtake all lines to the end of break down comment
    if {$CommentLine} \
    {
      if {!${LineBreak}} \
      {
        set CommentLine 0
      }
      continue
    }
          
    # We don't bother with empty rows either
    if {$CurrLine eq ""} \
    {
      continue
    }
  
    set KeepGoing 1
  
    while {$KeepGoing} \
    {
      # we find all lines with a \ at its end and add this lines together
      set LastCharacter [string index $CurrLine end]
   
      if {$LastCharacter eq "\\"} \
      {
        append FileContent [string range $CurrLine 0 end-1]
    
        gets $Fid CurrLine
      } \
      else \
      {
        # or append the last or a single line to FileContent
        append FileContent $CurrLine \n
        
        set KeepGoing 0
      }
    }
  }
  
  close $Fid
  
  return $FileContent
}


#---shows a window with unused translations
proc ::tk_msg::UnusedTranslations {} \
{
  ::tk_msg::LoadProc UnusedTranslationsSub "unused_translations.tcl"
 
  ::tk_msg::UnusedTranslationsSub
}

::tk_msg::Main
